"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const URL = require("url");
const request = require("request");
const clip_1 = require("./utils/clip");
const cleanup_title_1 = require("./utils/cleanup-title");
const html_entities_1 = require("html-entities");
const entities = new html_entities_1.AllHtmlEntities();
const client = require("cheerio-httpcli");
client.set('referer', false);
client.set('timeout', 10000);
exports.default = (url) => __awaiter(this, void 0, void 0, function* () {
    const res = yield client.fetch(url.href);
    if (res.error) {
        throw 'something happened';
    }
    const contentType = res.response.headers['content-type'];
    // HTMLじゃなかった場合は中止
    if (contentType.indexOf('text/html') === -1) {
        return null;
    }
    const $ = res.$;
    let title = $('meta[property="og:title"]').attr('content') ||
        $('meta[property="twitter:title"]').attr('content') ||
        $('title').text();
    if (title === undefined || title === null) {
        return null;
    }
    title = clip_1.default(entities.decode(title), 100);
    let image = $('meta[property="og:image"]').attr('content') ||
        $('meta[property="twitter:image"]').attr('content') ||
        $('link[rel="image_src"]').attr('href') ||
        $('link[rel="apple-touch-icon"]').attr('href') ||
        $('link[rel="apple-touch-icon image_src"]').attr('href');
    image = image ? URL.resolve(url.href, image) : null;
    const player = $('meta[property="twitter:player"]').attr('content');
    let description = $('meta[property="og:description"]').attr('content') ||
        $('meta[property="twitter:description"]').attr('content') ||
        $('meta[name="description"]').attr('content');
    description = description
        ? clip_1.default(entities.decode(description), 300)
        : null;
    if (title === description) {
        description = null;
    }
    let siteName = $('meta[property="og:site_name"]').attr('content') ||
        $('meta[name="application-name"]').attr('content') ||
        url.hostname;
    siteName = siteName ? entities.decode(siteName) : null;
    const favicon = $('link[rel="shortcut icon"]').attr('href') ||
        $('link[rel="icon"]').attr('href') ||
        '/favicon.ico';
    const find = (path) => new Promise(done => {
        const target = URL.resolve(url.href, path);
        request.head(target, (err, res) => {
            if (err) {
                done(null);
            }
            else if (res.statusCode == 200) {
                done(target);
            }
            else {
                done(null);
            }
        });
    });
    // 相対的なURL (ex. test) を絶対的 (ex. /test) に変換
    const toAbsolute = (relativeURLString) => {
        const relativeURL = URL.parse(relativeURLString);
        const isAbsolute = relativeURL.slashes || relativeURL.path[0] === '/';
        // 既に絶対的なら、即座に値を返却
        if (isAbsolute) {
            return relativeURLString;
        }
        // スラッシュを付けて返却
        return '/' + relativeURLString;
    };
    const icon = (yield find(favicon)) ||
        (
        // 相対指定を絶対指定に変換し再試行
        yield find(toAbsolute(favicon))) ||
        null;
    // Clean up the title
    title = cleanup_title_1.default(title, siteName);
    if (title === '') {
        title = siteName;
    }
    return {
        title: title || null,
        icon: icon || null,
        description: description || null,
        thumbnail: image || null,
        player: player || null,
        sitename: siteName || null
    };
});
